<?php

namespace Drupal\social_auth_yandex\Plugin\Network;

use Drupal\social_api\SocialApiException;
use Drupal\social_auth\Plugin\Network\NetworkBase;
use Drupal\social_auth_yandex\Settings\YandexAuthSettings;
use Yandex\OAuth\OAuthClient;

/**
 * Defines a Network Plugin for Social Auth Yandex.
 *
 * @package Drupal\social_auth_yandex\Plugin\Network
 *
 * @Network(
 *   id = "social_auth_yandex",
 *   social_network = "Yandex",
 *   type = "social_auth",
 *   handlers = {
 *     "settings": {
 *       "class": "\Drupal\social_auth_yandex\Settings\YandexAuthSettings",
 *       "config_id": "social_auth_yandex.settings"
 *     }
 *   }
 * )
 */
class YandexAuth extends NetworkBase implements YandexAuthInterface {

  /**
   * Sets the underlying SDK library.
   *
   * @return \Yandex\OAuth\OAuthClient
   *   The initialized 3rd party library instance.
   *
   * @throws \Drupal\social_api\SocialApiException
   *   If the SDK library does not exist.
   */
  protected function initSdk() {
    $class_name = OAuthClient::class;
    if (!class_exists($class_name)) {
      throw new SocialApiException(sprintf('The Yandex Library not found. Class: %s.', $class_name));
    }

    if ($this->validateConfig($this->settings)) {
      return new OAuthClient();
    }

    return FALSE;
  }

  /**
   * Checks that module is configured.
   *
   * @param \Drupal\social_auth_yandex\Settings\YandexAuthSettings $settings
   *   The Yandex auth settings.
   *
   * @return bool
   *   True if module is configured.
   *   False otherwise.
   */
  protected function validateConfig(YandexAuthSettings $settings) {
    $client_id = $settings->getClientId();
    $client_secret = $settings->getClientSecret();

    $is_valid = $client_id && $client_secret;

    if (!$is_valid) {
      $this->loggerFactory
        ->get('social_auth_yandex')
        ->error('Define Client ID and Client Secret on module settings.');
    }

    return $is_valid;
  }

}
